import sys
import json
from Command import Command
from PackageInfo import PackageInfo
from ClientLogger import ClientLogger
from ClientDaemon import ClientDaemon
from ClientOptionParser import ClientOptionParser


# noinspection PyClassHasNoInit
class TinkerAccessClient(object):

    # TODO: this can be static
    # noinspection PyMethodMayBeStatic
    def run(self):
        logger = ClientLogger.setup()
        (opts, args) = ClientOptionParser().parse_args()
        cmd = args[0].lower() if len(args) >= 1 and len(args[0].lower()) >= 1 else None
        command = Command(cmd)

        if command is not None:

            logger.debug(
                'Attempting to handle %s \'%s\' command with:\n%s',
                PackageInfo.pip_package_name, cmd,
                json.dumps(opts, indent=4, sort_keys=True)
            )

            try:
                if command is Command.START:
                    return ClientDaemon.start()

                elif command is Command.STOP:
                    return ClientDaemon.stop()

                elif command in [Command.RESTART, Command.RELOAD, Command.FORCE_RELOAD]:
                    return ClientDaemon.restart()

                elif command is Command.STATUS:
                    return ClientDaemon.status()

                else:
                    raise NotImplemented()

            except (KeyboardInterrupt, SystemExit):
                pass

            except Exception as e:
                logger.debug('%s \'%s\' command failed.', PackageInfo.pip_package_name, cmd)
                logger.exception(e)
                sys.exit(1)



#!/usr/bin/env python

### BEGIN INIT INFO
# Provides:          tinker-access-client
# Required-Start:    $remote_fs $syslog $network
# Required-Stop:     $remote_fs $syslog
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: tinker-access-client
# Description:       The tinker-access-client service is responsible for coordinating communication
# between RPi's modules (i.e. RFID reader, LCD, power relays etc..) and the remote tinker_access_server.
### END INIT INFO

from TinkerAccessClient import TinkerAccessClient

if __name__ == '__main__':
    TinkerAccessClient().run()



import os
import subprocess
from PackageInfo import PackageInfo
from ClientLogger import ClientLogger


class ServiceInstaller(object):
    @staticmethod
    def install(install_lib):
        logger = ClientLogger.setup(phase='install')
        logger.debug('Attempting to install the %s service...', PackageInfo.pip_package_name)
        try:
            update_script = '{0}{1}/scripts/update.sh'.format(install_lib, PackageInfo.python_package_name)
            install_script = '{0}{1}/scripts/install.sh'.format(install_lib, PackageInfo.python_package_name)
            service_script = '{0}{1}/Service.py'.format(install_lib, PackageInfo.python_package_name)
            os.chmod(update_script, 0755)
            os.chmod(install_script, 0755)
            os.chmod(service_script, 0755)

            # TODO: refactor to common utility, update...
            cmd = [install_script, '-evx']
            cwd = '{0}{1}/scripts/'.format(install_lib, PackageInfo.python_package_name)
            install_process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
            stdout_data, stderr_data = install_process.communicate()
            if install_process.returncode != 0:
                for ln in stderr_data.splitlines(True):
                    logger.error(ln)
                raise RuntimeError('%s service installation failed.' % PackageInfo.pip_package_name)
            else:
                for ln in stdout_data.splitlines(True):
                    logger.debug(ln)
        except Exception as e:
            logger.exception(e)
            raise e

        logger.debug('the %s service installation succeeded.', PackageInfo.pip_package_name)



import requests
from ClientLogger import ClientLogger


# TODO: add retry logic and automatically increase timeout on retries see:https://pypi.python.org/pypi/retrying
class LoggedRequest(object):

    @staticmethod
    def get(url, params=None, **kwargs):
        logger = ClientLogger.setup()
        logger.debug('Attempting API request...')

        try:
            response = requests.get(url, params, **kwargs)
            logger.debug('API response: %s', response.json())
            response.raise_for_status()

        except Exception as e:
            logger.debug('API request failed\n\turl: %s\n\tparams: %s\n\tkwargs: %s.', url, params, kwargs)
            logger.exception(e)
            raise e

        logger.debug('API request succeeded.')
        return response



# from tinker_access_client.tinker_access_client.ClientLogger import ClientLogger
# from tinker_access_client.ClientOptionParser import ClientOptionParser, ClientOption
from BaseHTTPServer import BaseHTTPRequestHandler

initial_pin_state = [
    [
        {'pin': 2, 'desc': '5v Power', 'status': False},
        {'pin': 4, 'desc': '5v Power', 'status': False},
        {'pin': 6, 'desc': 'Ground', 'status': False},
        {'pin': 8, 'desc': 'BCM 14', 'status': True},
        {'pin': 10, 'desc': 'BCM 15', 'status': True},
        {'pin': 12, 'desc': 'BCM 18', 'status': True},
        {'pin': 14, 'desc': 'Ground', 'status': False},
        {'pin': 16, 'desc': 'BCM 23', 'status': True},
        {'pin': 18, 'desc': 'BCM 24', 'status': True},
        {'pin': 20, 'desc': 'Ground', 'status': False},
        {'pin': 22, 'desc': 'BCM 25', 'status': True},
        {'pin': 24, 'desc': 'BCM 8', 'status': True},
        {'pin': 26, 'desc': 'BCM 7', 'status': True},
        {'pin': 28, 'desc': 'BCM 1', 'status': True},
        {'pin': 30, 'desc': 'Ground', 'status': False},
        {'pin': 32, 'desc': 'BCM 12', 'status': True},
        {'pin': 34, 'desc': 'Ground', 'status': False},
        {'pin': 36, 'desc': 'BCM 16', 'status': True},
        {'pin': 38, 'desc': 'BCM 20', 'status': True},
        {'pin': 40, 'desc': 'BCM 21', 'status': True}],
    [
        {'pin': 1, 'desc': '3v3 Power', 'status': False},
        {'pin': 3, 'desc': 'BCM 2', 'status': False},
        {'pin': 5, 'desc': 'BCM 3', 'status': False},
        {'pin': 7, 'desc': 'BCM 4', 'status': False},
        {'pin': 9, 'desc': 'Ground', 'status': False},
        {'pin': 11, 'desc': 'BCM 17', 'status': True},
        {'pin': 13, 'desc': 'BCM 27', 'status': True},
        {'pin': 15, 'desc': 'BCM 22', 'status': True},
        {'pin': 17, 'desc': '3v3 Power', 'status': False},
        {'pin': 19, 'desc': 'BCM 10', 'status': True},
        {'pin': 21, 'desc': 'BCM 9', 'status': True},
        {'pin': 23, 'desc': 'BCM 11', 'status': True},
        {'pin': 25, 'desc': 'Ground', 'status': False},
        {'pin': 27, 'desc': 'BCM 0', 'status': True},
        {'pin': 29, 'desc': 'BCM 5', 'status': True},
        {'pin': 31, 'desc': 'BCM 6', 'status': True},
        {'pin': 33, 'desc': 'BCM 13', 'status': True},
        {'pin': 35, 'desc': 'BCM 19', 'status': True},
        {'pin': 37, 'desc': 'BCM 26', 'status': True},
        {'pin': 39, 'desc': 'Ground', 'status': False}
    ]
]


class VirtualDeviceHandler(BaseHTTPRequestHandler):

    # def __init__(self):
    #     self.__pins = initial_pin_state[:]
    #     # self.__logger = ClientLogger.setup()
    #     # self.__logger.debug('New Handler created....')
    #     # self.__debug_port = ClientOptionParser().parse_args()[0].get(ClientOption.DEBUG_PORT)

    def do_GET(self):
        self.send_response(200)
        #raise NotImplemented

    def do_POST(self):
        self.send_response(200)
        #raise NotImplemented



import sys
import os
import ConfigParser
from optparse import OptionParser, OptionGroup

from Command import Command
from PackageInfo import PackageInfo
from ClientOption import ClientOption

ClientOptionDefaults = {
    ClientOption.DEBUG: False,
    ClientOption.DEBUG_PORT: 2020,
    ClientOption.CONFIG_FILE: '/etc/{0}.conf'.format(PackageInfo.pip_package_name),
    ClientOption.PID_FILE: '/var/run/{0}.pid'.format(PackageInfo.pip_package_name),
    ClientOption.LOG_FILE: '/var/log/{0}.log'.format(PackageInfo.pip_package_name),
    ClientOption.LOG_LEVEL: 10,
    ClientOption.SERVER_ADDRESS: 'http://10.2.1.2:5000',
    ClientOption.DEVICE_ID: 0,
    ClientOption.SERIAL_PORT_NAME: '/dev/ttyUSB0',
    ClientOption.SERIAL_PORT_SPEED: 9600,
    ClientOption.PIN_LOGOUT: 16,
    ClientOption.PIN_POWER_RELAY: 17,
    ClientOption.PIN_LED_RED: 21,
    ClientOption.PIN_LED_GREEN: 19,
    ClientOption.PIN_LED_BLUE: 20,
    ClientOption.PIN_CURRENT_SENSE: 12,
    ClientOption.LOGOUT_COAST_TIME: 7,
    ClientOption.RESTART_DELAY: 3,
    ClientOption.MAX_RESTART_ATTEMPTS: float('inf'),
    ClientOption.MINIMUM_UP_TIME: 5,
    ClientOption.LOG_ADDRESS_FOR_PAPER_TRAIL: None
}


class ClientOptionParser(object):
    def __init__(self, phase=None):
        self.__phase = phase
        self.__parser = OptionParser(
            version='%prog v{0}'.format(PackageInfo.version) if PackageInfo.version is not None else None)

        if phase == 'install':
            for arg in sys.argv:
                if str(arg).startswith('-'):
                    self.__parser.add_option(str(arg).split('=', 1)[0])

        usage = "\n%prog command [options]"
        commands = ['\n\ncommand:\n']
        for key, value in vars(Command).items():
            if not key.startswith('__'):
                desc = value['description']
                cmd = value['command']
                commands.append('\t%s : %s' % (cmd, desc))
        commands = '\n'.join(commands)
        usage += commands
        usage += '\n\nTinkerMill Raspberry Pi access control system.' \
                 '\n\nExamples:\n\n' \
                 '  Start the client with the device id set to \'plasma-cutter\'.' \
                 '\n  If the client were to stop unexpectedly, a restart would be attempted every 5 seconds.' \
                 '\n  The restart attempts would continue until the client stated successfully' \
                 ' and continued to run for least 10 seconds.' \
                 '\n\n  \'{0} --device-id=plasma-cutter --restart-delay=5 --max-restart-attempts=inf ' \
                 '--minimum-up-time=10\' ' \
                 '\n\n  Start the client configured to use a different tinker-access-server ' \
                 '(i.e. a development server) and an alternative serial port' \
                 '\n\n  \'{0} --server-address=http://<server-address> ' \
                 '--serial-port-name=/dev/ttyUSB1\' '.format(PackageInfo.python_package_name)

        self.__parser.set_usage(usage)

        self.__parser.add_option('--config-file',
                          help='the location of the config file to use [default:\'%default\'] '
                               'a non-default command-line option value will have precedence '
                               'over a config-file option value',
                          default=ClientOptionDefaults[ClientOption.CONFIG_FILE],
                          dest=ClientOption.CONFIG_FILE,
                          action='store')

        self.__parser.add_option('--debug',
                          help='run in foreground(a.k.a debug mode) [default:\'%default\']',
                          default=ClientOptionDefaults[ClientOption.DEBUG],
                          dest=ClientOption.DEBUG,
                          action='store_true')

        self.__parser.add_option('--debug-port',
                         help='The port to host the virtual device UI in debug mode [default:\'%default\']',
                         default=ClientOptionDefaults[ClientOption.DEBUG_PORT],
                         dest=ClientOption.DEBUG_PORT,
                         type='int',
                         action='store')

        self.__parser.add_option('--log-file',
                          help='the path and name of the log file [default:\'%default\']',
                          default=ClientOptionDefaults[ClientOption.LOG_FILE],
                          dest=ClientOption.LOG_FILE,
                          action='store')

        self.__parser.add_option('--pid-file',
                          help='the path & name of the pid file [default:\'%default\']',
                          default=ClientOptionDefaults[ClientOption.PID_FILE],
                          dest=ClientOption.PID_FILE,
                          action='store')

        self.__parser.add_option('--log-level',
                          help='the log level to use [default:%default]',
                          default=ClientOptionDefaults[ClientOption.LOG_LEVEL],
                          dest=ClientOption.LOG_LEVEL,
                          type='int',
                          action='store')

        self.__parser.add_option('--server-address',
                          help='the api\'s server address [default:\'%default\']',
                          default=ClientOptionDefaults[ClientOption.SERVER_ADDRESS],
                          dest=ClientOption.SERVER_ADDRESS,
                          action='store')

        self.__parser.add_option('--device-id',
                          help='the device id for this client [default:%default]',
                          default=ClientOptionDefaults[ClientOption.DEVICE_ID],
                          dest=ClientOption.DEVICE_ID,
                          action='store')

        self.__parser.add_option('--log-address-for-paper-trail',
                                 help='the log address for paper trail [default:%default]',
                                 default=ClientOptionDefaults[ClientOption.LOG_ADDRESS_FOR_PAPER_TRAIL],
                                 dest=ClientOption.LOG_ADDRESS_FOR_PAPER_TRAIL,
                                 action='store')

        resilience_group = OptionGroup(self.__parser, 'Resilience')

        resilience_group.add_option('--restart-delay',
                                    help='seconds to wait before attempting to re-start after a failure [default:%default]',
                                    default=ClientOptionDefaults[ClientOption.RESTART_DELAY],
                                    dest=ClientOption.RESTART_DELAY,
                                    type='int',
                                    action='store')

        resilience_group.add_option('--max-restart-attempts',
                                    help='the maximum number of times to attempt to re-start the client in the case '
                                         'that it quits unexpectedly [default:%default]',
                                    default=ClientOptionDefaults[ClientOption.MAX_RESTART_ATTEMPTS],
                                    dest=ClientOption.MAX_RESTART_ATTEMPTS,
                                    type='float',
                                    action='store')

        resilience_group.add_option('--minimum-up-time',
                                    help='the minimum number of seconds that the client must be running before considering '
                                         'it to have successfully started [default:%default]',
                                    default=ClientOptionDefaults[ClientOption.MINIMUM_UP_TIME],
                                    dest=ClientOption.MINIMUM_UP_TIME,
                                    type='int',
                                    action='store')

        resilience_group.add_option('--logout-coast-time',
                                    help='the maximum number of seconds to allow the physical machine '
                                         'to power down after logout [default:%default]',
                                    default=ClientOptionDefaults[ClientOption.LOGOUT_COAST_TIME],
                                    dest=ClientOption.LOGOUT_COAST_TIME,
                                    type='int',
                                    action='store')

        self.__parser.add_option_group(resilience_group)

        gpio_group = OptionGroup(self.__parser, 'RPi GPIO')

        gpio_group.add_option('--pin-logout',
                              help='the logout pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_LOGOUT],
                              dest=ClientOption.PIN_LOGOUT,
                              type='int',
                              action='store')

        gpio_group.add_option('--pin-power-relay',
                              help='the power relay pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_POWER_RELAY],
                              dest=ClientOption.PIN_POWER_RELAY,
                              type='int',
                              action='store')

        gpio_group.add_option('--pin-led-red',
                              help='the red led pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_LED_RED],
                              dest=ClientOption.PIN_LED_RED,
                              type='int',
                              action='store')

        gpio_group.add_option('--pin-led-green',
                              help='the green led pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_LED_GREEN],
                              dest=ClientOption.PIN_LED_GREEN,
                              type='int',
                              action='store')

        gpio_group.add_option('--pin-led-blue',
                              help='the blue led pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_LED_BLUE],
                              dest=ClientOption.PIN_LED_BLUE,
                              type='int',
                              action='store')

        gpio_group.add_option('--pin-current-sense',
                              help='the current sense pin [default:%default]',
                              default=ClientOptionDefaults[ClientOption.PIN_CURRENT_SENSE],
                              dest=ClientOption.PIN_CURRENT_SENSE,
                              type='int',
                              action='store')

        self.__parser.add_option_group(gpio_group)

        serial_group = OptionGroup(self.__parser, 'SERIAL')

        serial_group.add_option('--serial-port-name',
                                help='the serial port name to use [default:\'%default\']',
                                default=ClientOptionDefaults[ClientOption.SERIAL_PORT_NAME],
                                dest=ClientOption.SERIAL_PORT_NAME,
                                action='store')

        serial_group.add_option('--serial-port-speed',
                                help='the serial port speed to use [default:%default]',
                                default=ClientOptionDefaults[ClientOption.SERIAL_PORT_SPEED],
                                dest=ClientOption.SERIAL_PORT_SPEED,
                                type='int',
                                action='store')

        self.__parser.add_option_group(serial_group)

    def parse_args(self, args=None, values=None):
        (opts, args) = self.__parser.parse_args(args=args, values=values)
        items = vars(opts)

        options = self.__parser.option_list[:]
        for group in self.__parser.option_groups:
            options = options + group.option_list[:]

        if os.path.isfile(items.get(ClientOption.CONFIG_FILE)):
            config_file_parser = ConfigParser.RawConfigParser()
            config_file_parser.read(items.get(ClientOption.CONFIG_FILE))
            if config_file_parser.has_section('config'):
                for item in config_file_parser.items('config'):
                    option = next((i for i in options if i.dest == item[0]), None)
                    if option:
                        key = item[0]
                        value = item[1]
                        if option.type == 'int':
                            value = int(value)

                        if option.type == 'float':
                            value = float(value)

                        # prevents non-default command-line options from being replaced by config-file options
                        if items.get(key) == option.default != value:
                            items[key] = value

        # TODO: make args[0] required?, check the value and raise error parser.error()
        return items, args



class _VirtualSerial(object):
    def __init__(self, *args):
        pass

    # noinspection PyPep8Naming
    def flushInput(self, *args):
        pass

    # noinspection PyPep8Naming
    def flushOutput(self, *args):
        pass


# noinspection PyClassHasNoInit
class VirtualSerial:
    Serial = _VirtualSerial



import os
import sys

# TODO: convert to use LoggedPopen utility...
from subprocess import call, check_output

from Client import Client
from daemonize import Daemonize
from PackageInfo import PackageInfo
from ClientLogger import ClientLogger
from ClientOption import ClientOption
from ClientOptionParser import ClientOptionParser


# noinspection PyClassHasNoInit
class ClientDaemon:
    @staticmethod
    def start():
        # TODO: handle if process already running, can't lock on pid_file
        logger = ClientLogger.setup()
        opts = ClientOptionParser().parse_args()[0]
        pid_file = opts.get(ClientOption.PID_FILE)
        foreground = opts.get(ClientOption.DEBUG)
        if not os.path.isfile(pid_file):
            logger.debug('Attempting to start the %s daemon...', PackageInfo.pip_package_name)
            try:
                client = Client()
                daemon = Daemonize(
                    app=PackageInfo.pip_package_name, pid=pid_file,
                    action=client.run,
                    foreground=foreground, verbose=True,
                    logger=logger
                )
                daemon.start()
            except Exception as e:
                logger.debug('%s daemon start failed.', PackageInfo.pip_package_name)
                logger.exception(e)
                raise e

    @staticmethod
    def stop():
        logger = ClientLogger.setup()
        opts = ClientOptionParser().parse_args()[0]
        pid_file = opts.get(ClientOption.PID_FILE)
        if os.path.isfile(pid_file): # TODO: maybe use while loop and timeout
            logger.debug('Attempting to stop the %s daemon...', PackageInfo.pip_package_name)
            try:
                # TODO: use call_output return, and set timeout, including logging etc..
                # should use a common util for when we need to kill a process like this

                # Also needs some work if the process is not already running
                call(['pkill', '-F', pid_file])
            except Exception as e:
                logger.debug('%s daemon stop failed.', PackageInfo.pip_package_name)
                logger.exception(e)
                raise e

    @staticmethod
    def restart():
        # TODO: restart should wait until the client is idle.. not in use...

        logger = ClientLogger.setup()
        logger.debug('Attempting to restart the %s daemon...', PackageInfo.pip_package_name)
        try:
            ClientDaemon.stop()
            ClientDaemon.start()

        except Exception as e:
            logger.debug('%s daemon restart failed.', PackageInfo.pip_package_name)
            logger.exception(e)
            raise e

    @staticmethod
    def status():
        logger = ClientLogger.setup()
        opts = ClientOptionParser().parse_args()[0]
        pid_file = opts.get(ClientOption.PID_FILE)
        logger.debug('Attempting to check the %s daemon status...', PackageInfo.pip_package_name)
        try:
            # TODO: use call_output return, and set timeout, including logging etc..
            # should use a common util for when we need to kill a process like this

            # Also needs some work if the process is not already running
            # TODO: can't seem to catch the exception here?
            FNULL = open(os.devnull, 'w')
            return_code = check_output(['pgrep', '-F', pid_file], stderr=FNULL)
            if return_code != 0:
                sys.stdout.write('running\n')
                sys.stdout.flush()
                return
        except Exception:
            pass
        # except CalledProcessError:
        #     pass
        # except Exception as e:
        #     pass
        #     # logger.debug('%s daemon status failed.', PackageInfo.pip_package_name)
        #     # logger.exception(e)
        #     # raise e
        sys.exit(1)



class Command(object):
    START = dict(command='start', description='start desc...')
    STOP = dict(command='stop', description='stop desc...')
    STATUS = dict(command='status', description='status desc...')
    UPDATE = dict(command='update', description='update...')
    RESTART = dict(command='restart', description='restart...')
    RELOAD = dict(command='reload', description='reload...')
    # REMOVE = dict(command='remove', description='remove...')
    # UNINSTALL = dict(command='uninstall', description='uninstall...')
    # RECONFIGURE = dict(command='re-configure', description='re-configure...')
    FORCE_RELOAD = dict(command='force_reload', description='force_reload...')

    def __new__(self, command):
        for key, value in vars(Command).items():
            if not key.startswith('__'):
                if command == value['command']:
                    return value
        return command


import json
from ClientLogger import ClientLogger
from ClientOptionParser import ClientOption


class Channel(object):
    LCD, SERIAL, LED, PIN = range(0, 4)

    def __new__(cls, channel):
        for key, value in vars(Channel).items():
            if not key.startswith('__'):
                if value == channel:
                    return key
        return None


class DeviceApi(object):
    def __init__(self, opts):
        self.__opts = opts
        self.__logger = ClientLogger.setup()
        self.__logger.debug('Attempting device initialization...')
        try:

            # !Important: These modules are imported locally at runtime so that the unit test
            # can run on non-rpi devices where GPIO won't load and cannot work
            # (i.e. the build server, and dev environments) see: test_DeviceApi.py
            self.__init__device_modules()

        except (RuntimeError, ImportError) as e:
            self.__logger.exception(e)
            raise e

        except Exception as e:
            self.__logger.debug('Device initialization failed with %s.', json.dumps(opts, indent=4, sort_keys=True))
            self.__logger.exception(e)
            raise e

        self.__logger.debug('Device initialization succeeded.')

    # TODO: re-configure to use edge detection for signaling rather than polling
    # noinspection PyPep8Naming,PyUnresolvedReferences
    def __init__device_modules(self):

        try:
            import RPi.GPIO
            import lcdModule.LCD
            import serial
        except (RuntimeError, ImportError) as e:
            # If the above import fails it means this code is not executing on a physical RPi device.
            # so the modules are replaced with virtual modules for testing/development/debug purposes
            # this logic will probably get moved into a CustomImporter module eventually
            # see file://tinkerAccess/tinker_access_client/tests/README.md for more info
            try:
                from debug.VirtualRPi import VirtualRPi as RPi
                from debug.VirtualLcd import VirtualLcd as lcdModule
                from debug.VirtualSerial import VirtualSerial as serial
            except Exception as ex:
                self.__logger.debug('Failed to patch RPi device modules with virtual device modules.')
                self.__logger.exception(ex)
                raise e

        GPIO = RPi.GPIO
        GPIO.setmode(GPIO.BCM)

        # TODO: remove once lcdModule is fixed fixed to not also call cleanup
        GPIO.cleanup()
        GPIO.setWarnings(False)

        GPIO.setup(self.__opts.get(ClientOption.PIN_LED_RED), GPIO.OUT)
        GPIO.setup(self.__opts.get(ClientOption.PIN_LED_BLUE), GPIO.OUT)
        GPIO.setup(self.__opts.get(ClientOption.PIN_LED_GREEN), GPIO.OUT)
        GPIO.setup(self.__opts.get(ClientOption.PIN_POWER_RELAY), GPIO.OUT)
        GPIO.setup(self.__opts.get(ClientOption.PIN_LOGOUT), GPIO.IN, GPIO.PUD_DOWN)
        GPIO.setup(self.__opts.get(ClientOption.PIN_CURRENT_SENSE), GPIO.IN, GPIO.PUD_DOWN)
        self.__GPIO = GPIO

        # TODO: lcdModule needs some love... I'll come back to this later can probably just completely remove it.
        LCD = lcdModule.LCD
        LCD.lcd_init()
        self.__LCD = LCD

        serial_port_name = self.__opts.get(ClientOption.SERIAL_PORT_NAME)
        serial_port_speed = self.__opts.get(ClientOption.SERIAL_PORT_SPEED)
        self.__serial_connection = serial.Serial(serial_port_name, serial_port_speed)
        self.__serial_connection.flushInput()
        self.__serial_connection.flushOutput()

    def write(self, channel, *args):
        channel_name = Channel(channel)
        self.__logger.debug('Attempting to write to \'%s\' with args %s...', channel_name, args)

        try:
            if channel == Channel.LED:
                red = len(args) >= 1 and args[0] is True
                green = len(args) >= 2 and args[1] is True
                blue = len(args) >= 3 and args[2] is True
                self.__write_to_led(red, green, blue)

            elif channel == Channel.LCD:
                first_line = args[0] if len(args) >= 1 else ''
                second_line = args[1] if len(args) >= 2 else ''
                self.__write_to_lcd(first_line, second_line)

            elif channel == Channel.PIN:
                pin = args[0] if len(args) >= 1 else None
                state = args[1] if len(args) >= 2 else None
                self.__write_to_pin(pin, state)

            else:
                raise NotImplemented

        except Exception as e:
            self.__logger.debug('Write to \'%s\' failed.', channel_name, args)
            self.__logger.exception(e)
            raise e

        self.__logger.debug('Write to \'%s\' succeeded.', channel_name)

    # noinspection PyPep8Naming
    def __write_to_led(self, red, green, blue):
        GPIO = self.__GPIO
        GPIO.output(self.__opts.get(ClientOption.PIN_LED_RED), red)
        GPIO.output(self.__opts.get(ClientOption.PIN_LED_GREEN), green)
        GPIO.output(self.__opts.get(ClientOption.PIN_LED_BLUE), blue)

    # noinspection PyPep8Naming
    def __write_to_lcd(self, first_line, second_line):
        LCD = self.__LCD
        LCD.lcd_string(first_line, LCD.LCD_LINE_1)
        LCD.lcd_string(second_line, LCD.LCD_LINE_2)

    # noinspection PyPep8Naming
    def __write_to_pin(self, pin, state):
        GPIO = self.__GPIO
        state = GPIO.LOW if not state else GPIO.HIGH
        GPIO.output(pin, state)

    def read(self, channel, *args):
        channel_name = Channel(channel)
        self.__logger.debug('Attempting to read from \'%s\' with \'%s\'...', channel_name, args)

        try:

            if channel == Channel.SERIAL:
                value = self.__read_from_serial()

            elif channel == Channel.PIN:
                pin = args[0] if len(args) >= 1 else None
                expected_state = args[1] if len(args) >= 2 else True
                value = self.__read_from_pin(pin, expected_state)

            else:
                raise NotImplemented

        except Exception as e:
            self.__logger.debug('Read from \'%s\' failed with args \'%s\'.', channel_name, args)
            self.__logger.exception(e)
            raise e

        if value is not None and value is not False:
            self.__logger.debug('Successfully read \'%s\' from \'%s\'.', value, channel_name)

        return value

    def __read_from_serial(self):
        serial_connection = self.__serial_connection

        if serial_connection.inWaiting() > 1:
            value = serial_connection.readline().strip()[-12:]
            serial_connection.flushInput()
            serial_connection.flushOutput()
            return value

        return None

    # noinspection PyPep8Naming
    def __read_from_pin(self, pin, expected_state):
        GPIO = self.__GPIO
        expected_state = GPIO.LOW if not expected_state else GPIO.HIGH
        return GPIO.input(pin) == expected_state



import os
import logging
import logging.handlers as handlers
from PackageInfo import PackageInfo
from ContextFilter import ContextFilter
from ClientOptionParser import ClientOptionParser, ClientOption


class ClientLogger(object):

    @staticmethod
    def setup(phase=None):
        logger = logging.getLogger(PackageInfo.python_package_name)
        (opts, args) = ClientOptionParser(phase=phase).parse_args()

        logger.filters = []
        logger.handlers = []

        debug_mode = opts.get(ClientOption.DEBUG)
        log_level = opts.get(ClientOption.LOG_LEVEL)
        if debug_mode:
            logger.setLevel(10)
        else:
            logger.setLevel(log_level if log_level else 40)

        logger.addFilter(ContextFilter(opts))

        sys_log_format = '%(asctime)s %(hostname)s %(app_id)s %(version)s %(device_id)s %(message)s'
        sys_log_formatter = logging.Formatter(sys_log_format, datefmt='%FT%T')

        # paperTrail
        log_address_for_paper_trail = opts.get(ClientOption.LOG_ADDRESS_FOR_PAPER_TRAIL)
        if not debug_mode and log_address_for_paper_trail:
            (paper_trail_host, paper_trail_port) = log_address_for_paper_trail.split(':', 2)
            paper_trail_handler = handlers.SysLogHandler((paper_trail_host, int(paper_trail_port)))
            paper_trail_handler.setFormatter(sys_log_formatter)
            logger.addHandler(paper_trail_handler)

        # dev/log
        sys_log_file = '/var/log/syslog'
        if not debug_mode and os.path.exists(sys_log_file):
            sys_log_handler = handlers.SysLogHandler('/dev/log')
            logger.addHandler(sys_log_handler)
            sys_log_handler.setFormatter(sys_log_formatter)

        # console
        logger.addHandler(logging.StreamHandler())

        # fileHandler
        log_file = opts.get(ClientOption.LOG_FILE)
        if log_file:
            if debug_mode:
                log_file = '{0}.debug'.format(log_file)

            if not os.path.exists(os.path.dirname(log_file)):
                os.makedirs(os.path.dirname(log_file))

            file_handler = handlers.TimedRotatingFileHandler(log_file, when='D', interval=1, backupCount=7)
            logger.addHandler(file_handler)

        return logger



# noinspection PyClassHasNoInit
class ClientOption:
    DEBUG = 'debug'
    PID_FILE = 'pid_file'
    LOG_FILE = 'log_file'
    LOG_LEVEL = 'log_level'
    DEVICE_ID = 'device_id'
    DEBUG_PORT = 'debug_port'
    PIN_LOGOUT = 'pin_logout'
    CONFIG_FILE = 'config_file'
    PIN_LED_RED = 'pin_led_red'
    PIN_LED_BLUE = 'pin_led_blue'
    RESTART_DELAY = 'restart_delay'
    PIN_LED_GREEN = 'pin_led_green'
    SERVER_ADDRESS = 'server_address'
    PIN_POWER_RELAY = 'pin_power_relay'
    MINIMUM_UP_TIME = 'minimum_up_time'
    SERIAL_PORT_NAME = 'serial_port_name'
    SERIAL_PORT_SPEED = 'serial_port_speed'
    PIN_CURRENT_SENSE = 'pin_current_sense'
    LOGOUT_COAST_TIME = 'logout_coast_time'
    MAX_RESTART_ATTEMPTS = 'max_restart_attempts'
    LOG_ADDRESS_FOR_PAPER_TRAIL = 'log_address_for_paper_trail'



import time

from PackageInfo import PackageInfo
from ClientLogger import ClientLogger
from DeviceApi import DeviceApi, Channel
from ClientOptionParser import ClientOptionParser


class Client(object):

    def __init__(self):
        self.__logger = ClientLogger.setup()
        self.__opts = ClientOptionParser().parse_args()[0]
        self.__deviceApi = DeviceApi(self.__opts)

    def run(self):
        try:
            wait = 5
            counter = 0
            while counter < 720:
                counter += 1
                time.sleep(wait)
                self.__logger.debug('%s running for %s seconds...', PackageInfo.pip_package_name, counter * wait)

        except KeyboardInterrupt as e:
            self.__logger.debug('Keyboard-Interrupt')
            raise e

        except SystemExit as e:
            self.__logger.debug('System-Exit, process killed')
            raise e

        except Exception as e:
            self.__logger.debug('Unable to start the %s daemon.')
            self.__logger.exception(e)
            raise e



import socket
import logging
from PackageInfo import PackageInfo
from ClientOptionParser import ClientOption


class ContextFilter(logging.Filter):
    def __init__(self, opts):
        self.__hostname = socket.gethostname()
        self.__device_id = opts.get(ClientOption.DEVICE_ID)
        self.__app_id = PackageInfo.pip_package_name
        self.__version = PackageInfo.version

    def filter(self, record):
        record.hostname = self.__hostname
        record.app_id = self.__app_id
        record.device_id = self.__device_id
        record.version = self.__version

        # TODO: be nice to have action, user_id, badge_id etc..
        record.user_id = None
        record.badge_id = None
        record.user_name = None
        record.action = None

        return True
